@@ -13,7 +13,7 @@ use Carp 'croak';
 use Net::SIP::Debug;
 use Net::SIP::Util ':all';
 use Digest::MD5 'md5_hex';
-use fields qw( realm opaque user2pass user2a1 i_am_proxy dispatcher );
+use fields qw( realm user2pass i_am_proxy dispatcher );
 
 ###########################################################################
 # creates new Authorize object
@@ -30,12 +30,7 @@ sub new {
 	my ($class,%args) = @_;
 	my $self = fields::new( $class );
 	$self->{realm} = $args{realm} || 'p5-net-sip';
-	$self->{opaque} = $args{opaque};
-
-	$args{user2pass} || $args{user2a1} || croak 'no user2pass or user2a1 known';
-
-	$self->{user2pass} = $args{user2pass};
-	$self->{user2a1} = $args{user2a1};
+	$self->{user2pass} = $args{user2pass} || croak 'no user2pass known';
 	$self->{i_am_proxy} = $args{i_am_proxy};
 	$self->{dispatcher} = $args{dispatcher} || croak 'no dispatcher';
 	return $self;
@@ -47,7 +42,7 @@ sub new {
 #  $packet: Net::SIP::Request
 #  $leg: Net::SIP::Leg where request came in (and response gets send out)
 #  $addr: ip:port where request came from and response will be send
-# Returns: TRUE if it handled the packet
+# Returns: TRUE if it handled the packet 
 ###########################################################################
 sub receive {
 	my Net::SIP::Authorize $self = shift;
@@ -58,7 +53,6 @@ sub receive {
 		DEBUG( 100,"pass thru response" );
 		return;
 	}
-	my $method = $packet->method;
 
 	# check authorization on request
 	my ($rq_key,$rs_key,$acode) = $self->{i_am_proxy}
@@ -67,14 +61,13 @@ sub receive {
 		;
 	my @auth = $packet->get_header( $rq_key );
 	my $user2pass = $self->{user2pass};
-	my $user2a1 = $self->{user2a1};
 	my $realm = $self->{realm};
-	my $opaque = $self->{opaque};
 
 	# there might be multiple auth, pick the right realm
 	my (@keep_auth,$authorized);
-
 	foreach my $auth ( @auth ) {
+
+
 		# RFC 2617
 		my ($data,$param) = sip_hdrval2parts( $rq_key => $auth );
 		if ( $param->{realm} ne $realm ) {
@@ -82,17 +75,7 @@ sub receive {
 			push @keep_auth,$auth;
 			next;
 		}
-		if ( defined $opaque ) {
-			if ( ! defined $param->{opaque} ) {
-				DEBUG( 10,"expected opaque value, but got nothing" );
-				next;
-			} elsif ( $param->{opaque} ne $opaque ) {
-				DEBUG( 10,"got wrong opaque value '$param->{opaque}', expected '$opaque'" );
-				next;
-			}
-		}
-
-		my ($user,$nonce,$uri,$resp,$qop,$cnonce,$algo ) =
+		my ($user,$nonce,$uri,$resp,$qop,$cnonce,$algo ) = 
 			@{$param}{ qw/ username nonce uri response qop cnonce algorithm / };
 		if ( lc($data) ne 'digest'
 			|| ( $algo && lc($algo) ne 'md5' )
@@ -102,61 +85,33 @@ sub receive {
 		};
 
 		# we support with and w/o qop
-		# get a1_hex from either user2a1 or user2pass
-		my $a1_hex;
-		if ( ref($user2a1)) {
-			if ( ref($user2a1) eq 'HASH' ) {
-				$a1_hex = $user2a1->{$user}
-			} else {
-				$a1_hex = invoke_callback( $user2a1,$user,$realm );
-			}
-		}
-		if ( ! defined($a1_hex) && ref($user2pass)) {
-			my $pass;
-			if ( ref($user2pass) eq 'HASH' ) {
-				$pass = $user2pass->{$user}
-			} else {
-				$pass = invoke_callback( $user2pass,$user );
-			}
-			# if wrong credentials ask again for authorization
-			last if ! defined $pass;
-			$a1_hex = md5_hex(join( ':',$user,$realm,$pass ));
+		my $pass = ref($user2pass) eq 'HASH' ? $user2pass->{$user} 
+			: invoke_callback( $user2pass,$user );
+		my $a1 = join( ':',$user,$realm,$pass );
+		my $a2 = join( ':',$packet->method,$uri );
+
+		my $want_response;
+		if ( $qop ) {
+			# 3.2.2.1
+			$want_response = md5_hex( join( ':',
+				md5_hex($a1),
+				$nonce,
+				1,
+				$cnonce,
+				$qop,
+				md5_hex($a2)
+			));
+		} else {
+			 # 3.2.2.1 compability with RFC2069
+			 $want_response = md5_hex( join( ':',
+			 	md5_hex($a1),
+				$nonce,
+				md5_hex($a2)
+			));
 		}
 
-		# ACK just reuse the authorization from INVITE, so they should
-		# be checked against method INVITE
-		# for CANCEL the RFC doesn't say anything, so we assume it uses
-		# CANCEL but try INVITE if this fails
-		my @a2 = 
-			$method eq 'ACK' ? ("INVITE:$uri") :
-			$method eq 'CANCEL' ? ("CANCEL:$uri","INVITE:$uri") :
-			("$method:$uri");
-
-		while (my $a2 = shift(@a2)) {
-			my $want_response;
-			if ( $qop ) {
-				# 3.2.2.1
-				$want_response = md5_hex( join( ':',
-					$a1_hex,
-					$nonce,
-					1,
-					$cnonce,
-					$qop,
-					md5_hex($a2)
-				));
-			} else {
-				 # 3.2.2.1 compability with RFC2069
-				 $want_response = md5_hex( join( ':',
-					$a1_hex,
-					$nonce,
-					md5_hex($a2)
-				));
-			}
-
-			if ( $resp eq $want_response ) {
-				$authorized = 1;
-				last;
-			}
+		if ( $resp eq $want_response ) {
+			$authorized = 1;
 		}
 	}
 
@@ -169,16 +124,9 @@ sub receive {
 		return;
 	}
 
-	# CANCEL or ACK cannot be prompted for authorization, so
-	# they should provide the right data already
-	# unauthorized CANCEL or ACK are only valid as response to
-	# 401/407 from this Authorize, so they should not be propagated
-	return $acode if $method eq 'CANCEL' or $method eq 'ACK'; 
-
 	# not authorized yet, ask to authenticate
 	# keep it simple RFC2069 style
 	my $digest = qq[Digest algorithm=MD5, realm="$realm",].
-		( defined($opaque) ? qq[ opaque="$opaque",] : '' ).
 		' nonce="'. md5_hex( $realm.rand(2**32)).'"';
 
 	my $resp = $packet->create_response(
@@ -188,7 +136,7 @@ sub receive {
 	);
 
 	$self->{dispatcher}->deliver( $resp, leg => $leg, dst_addr => $addr );
-
+	
 	# return $acode (TRUE) to show that packet should
 	# not passed thru
 	return $acode;
